/*
 * Display routines for text mode.
 *
 */
#include <stdio.h>
#include <math.h>

/* ======================================================================== */

void text_vu_headers() {
/*
 * Purpose: displays the VU meter headers in text mode.  General appearance
 *          is modelled after the following.
 *
 *   *********** Left ***********   *********** Right **********
 *   |inf -40  -20 -12 -8  -4   0   |inf -40  -20 -12 -8  -4   0     
 *   | |   |    |   |   |   |   |   | |   |    |   |   |   |   |
 *   |===============  #         !! |=========   #              !!
 *
 */
  fprintf(stderr,"*********** Left ***********   *********** Right **********\n");
  fprintf(stderr,"|inf -40  -20 -12 -8  -4   0   |inf -40  -20 -12 -8  -4   0\n");
  fprintf(stderr,"| |   |    |   |   |   |   |   | |   |    |   |   |   |   |\n");
}
/* ======================================================================== */

int text_vu_length(float value) {
/*
 * Purpose: calculates the length of the bar required for the given value.
 *
 */
int len = 0;
  if (value >= -4) {
    len = (21 + (int)rint((value+4)*4/4.0));
  } else
  if (value >= -8) {
    len = (17 + (int)rint((value+8)*4/4.0));
  } else
  if (value >= -12) {
    len = (13 + (int)rint((value+12)*4/4.0));
  } else
  if (value >= -20) {
    len = (9 + (int)rint((value+20)*4/8.0));
  } else
  if (value >= -40) {
    len = (4 + (int)rint((value+40)*5/20.0));
  } else {
    len = (int)rint((value+96)*4/56.0);
  }
  return(len);
}
/* ======================================================================== */

void text_vu_values(float left_max, float right_max, float left_ave, float right_ave) {
/*
 * Purpose: plots the VU meter readings, given in dB.  A "clip" is considered
 *          to be 3 consecutive samples of 0dB.
 */
static int over_count[2] = {0,0};
char buf[40];
int channel;
int c,i;
float max_value, ave_value;

  buf[0] = '|';
  fprintf(stderr,"\r");
  for (channel=0;channel<2;channel++) {
    if (channel==0) {
      max_value = left_max;
      ave_value = left_ave;
    } else {
      max_value = right_max;
      ave_value = right_ave;
    }
    c = 2;
    if (max_value > 0) 
      max_value = 0;
    if (ave_value > 0)
      ave_value = 0;
    if (max_value == 0)
      over_count[channel]++;
    else
      over_count[channel] = 0;
    c = 2 + text_vu_length(ave_value);

    for (i=0;i<c;i++)
      buf[i+1] = '=';
    for (i=c;i<27;i++)
      buf[i+1] = ' ';
    buf[text_vu_length(max_value)+2] = '#';
    buf[27+1] = '\0';
    fprintf(stderr,buf);

    if (over_count[channel] > 0)
      fprintf(stderr,"!! ");
    else
      fprintf(stderr,"   ");
  }
  fflush(stdout);
}
/* ======================================================================== */
